@@ -134,7 +134,7 @@ def grouped(iterable: Iterable, n: int, *, keep_tail=False) -> List[tuple]:
 
     if not keep_tail:
         return list(zip(*[iter(iterable)] * n))
-    with batch_manager(object(), iterable, batch_size=n) as manager:
+    with batch_manager(iterable, batch_size=n, max_batch_size=n) as manager:
         return [tuple(batch) for batch in manager]
 
 
@@ -338,7 +338,7 @@ def allclose(*arrays: np.ndarray, **kwargs) -> bool:
     allclose : bool
 
     """
-    for i, arr in enumerate(arrays[:1]):
+    for i, arr in enumerate(arrays[:-1]):
         if not np.allclose(arr, arrays[i + 1], **kwargs):
             return False
     return True
@@ -887,7 +887,11 @@ class Saving(LoggingMixin):
             return False
         if callable(elem):
             return False
-        return True
+        try:
+            json.dumps({"": elem})
+            return True
+        except TypeError:
+            return False
 
     @staticmethod
     def _check_list_and_tuple(arr: Union[list, tuple]):
@@ -1536,8 +1540,7 @@ class batch_manager(context_error_handler):
 
     Examples
     --------
-    >>> instance = type("test", (object,), {})()
-    >>> with batch_manager(instance, np.arange(5), np.arange(1, 6), batch_size=2) as manager:
+    >>> with batch_manager(np.arange(5), np.arange(1, 6), batch_size=2) as manager:
     >>>     for arr, tensor in manager:
     >>>         print(arr, tensor)
     >>>         # Will print:
@@ -1547,11 +1550,15 @@ class batch_manager(context_error_handler):
 
     """
 
-    def __init__(self, *inputs, n_elem=1e6, batch_size=None, max_batch_size=1024):
+    def __init__(self,
+                 *inputs,
+                 n_elem: int = 1e6,
+                 batch_size: int = None,
+                 max_batch_size: int = 1024):
         if not inputs:
             raise ValueError("inputs should be provided in general_batch_manager")
         input_lengths = list(map(len, inputs))
-        self._n, self._rs, self._inputs = input_lengths[0], [], inputs
+        self._n, self._inputs = input_lengths[0], inputs
         assert all(length == self._n for length in input_lengths), "inputs should be of same length"
         if batch_size is not None:
             self._batch_size = batch_size
