@@ -277,7 +277,8 @@ def register_metric(name, sign, requires_prob):
     return _register
 
 
-estimate_fn_type = Callable[[np.ndarray], np.ndarray]
+generic_data_type = Union[np.ndarray, Any]
+estimate_fn_type = Callable[[generic_data_type], np.ndarray]
 scoring_fn_type = Callable[[List[float], float, float], float]
 collate_fn_type = Callable[[List[np.ndarray], bool], np.ndarray]
 predict_method_type = Union[estimate_fn_type, None]
@@ -381,8 +382,8 @@ class Estimator(LoggingMixin):
         return Statistics("\n".join(msg_list), statistics)
 
     def estimate(self,
-                 x: np.ndarray,
-                 y: np.ndarray,
+                 x: generic_data_type,
+                 y: generic_data_type,
                  methods: Dict[str, Union[estimate_fn_type, List[estimate_fn_type]]],
                  *,
                  scoring_function: Union[str, scoring_fn_type] = "default",
@@ -399,17 +400,27 @@ class Estimator(LoggingMixin):
         self.log_block_msg(statistics.msg, self.info_prefix, "Results", verbose_level)
         return statistics.data
 
+    def select(self,
+               method_names: List[str]) -> "Estimator":
+        new_estimator = Estimator(
+            self.type,
+            verbose_level=self._verbose_level,
+            metric_config=self._metric.config
+        )
+        new_estimator.raw_metrics = {k: self.raw_metrics[k] for k in method_names}
+        new_estimator.final_scores = {k: self.final_scores[k] for k in method_names}
+        return new_estimator
+
     @classmethod
     def merge(cls, estimators: List["Estimator"]) -> "Estimator":
-        first_estimator = estimators[0]
-        new_final_scores = {key: [value] for key, value in first_estimator.final_scores.items()}
-        new_raw_metrics = {key: [value] for key, value in first_estimator.raw_metrics.items()}
-        for estimator in estimators[1:]:
+        new_raw_metrics, new_final_scores = {}, {}
+        for estimator in estimators:
             for key, value in estimator.raw_metrics.items():
-                new_raw_metrics[key].append(value)
-                new_final_scores[key].append(estimator.final_scores[key])
+                new_raw_metrics.setdefault(key, []).append(value)
+                new_final_scores.setdefault(key, []).append(estimator.final_scores[key])
         new_raw_metrics = {k: np.concatenate(v) for k, v in new_raw_metrics.items()}
         new_final_scores = {k: sum(v) / len(v) for k, v in new_final_scores.items()}
+        first_estimator = estimators[0]
         first_metric_ins = first_estimator._metric
         new_estimator = cls(
             first_metric_ins.type,
@@ -428,7 +439,7 @@ class PatternBase(ABC):
         pass
 
     def predict(self,
-                x: np.ndarray,
+                x: Union[np.ndarray, Any],
                 *,
                 requires_prob: bool = False) -> np.ndarray:
         predict_method = self.predict_method(requires_prob)
@@ -498,7 +509,10 @@ class ModelPattern(PatternBase, LoggingMixin):
         predict_method = self._predict_prob_method if requires_prob else self._predict_method
         if isinstance(predict_method, str):
             if self.model is None:
-                raise ValueError("Either init_method or Callable predict_method is required in ModelPattern")
+                raise ValueError(
+                    "Either init_method or Callable predict_method is required in ModelPattern "
+                    f"(requires_prob={requires_prob})"
+                )
             predict_method = getattr(self.model, predict_method, None)
         return predict_method
 
@@ -561,7 +575,9 @@ class EnsemblePattern(PatternBase):
         predictions = np.array(arrays)
         if not requires_prob and np.issubdtype(predictions.dtype, np.integer):
             max_class = predictions.max() + 1
-            predictions = predictions.squeeze(2).T
+            if len(predictions.shape) == 3:
+                predictions = predictions.squeeze(2)
+            predictions = predictions.T
             counts = np.apply_along_axis(partial(np.bincount, minlength=max_class), 1, predictions)
             return counts.argmax(1).reshape([-1, 1])
         return predictions.mean(0)
@@ -662,7 +678,7 @@ class Comparer(LoggingMixin):
 
     def log_statistics(self,
                        verbose_level: int = 1,
-                       **kwargs) -> None:
+                       **kwargs) -> str:
         sorted_metrics = sorted(self.estimator_statistics)
         body = {}
         same_choices: choices_type = None
@@ -758,10 +774,9 @@ class Comparer(LoggingMixin):
         single_split = "-" * length
         double_split = "=" * length
         main_msg = f"\n{single_split}\n".join(msgs)
-        self.log_block_msg(
-            f"{double_split}\n{main_msg}\n{double_split}",
-            self.info_prefix, "Results", verbose_level
-        )
+        final_msg = f"{double_split}\n{main_msg}\n{double_split}"
+        self.log_block_msg(final_msg, self.info_prefix, "Results", verbose_level)
+        return final_msg
 
     def compare(self,
                 x: np.ndarray,
@@ -807,6 +822,17 @@ class Comparer(LoggingMixin):
         self.log_statistics(verbose_level, **kwargs)
         return self
 
+    def select(self,
+               method_names: List[str]) -> "Comparer":
+        new_patterns = {}
+        for name in method_names:
+            pattern = self.patterns.get(name)
+            if pattern is None:
+                raise ValueError(f"'{name}' is not found in current patterns")
+            new_patterns[name] = pattern
+        new_estimators = [estimator.select(method_names) for estimator in self.estimators.values()]
+        return Comparer(new_patterns, new_estimators, verbose_level=self._verbose_level)
+
     @classmethod
     def merge(cls,
               comparer_list: List["Comparer"],
@@ -1109,11 +1135,11 @@ class Tracker:
         exists = os.path.isdir(self.log_folder)
         if exists:
             if not overwrite:
-                print(f"loading tracker from '{self.log_folder}'")
+                print(f"{LoggingMixin.info_prefix}loading tracker from '{self.log_folder}'")
                 self._load()
             else:
                 print(f"{LoggingMixin.warning_prefix}'{self.log_folder}' already exists, it will be overwritten")
-                shutil.rmtree(self.log_folder)
+                self.clear(confirm=False)
         if not exists or overwrite:
             os.makedirs(self.log_folder)
             self.reset()
@@ -1131,6 +1157,12 @@ class Tracker:
         os.makedirs(folder, exist_ok=True)
         return folder
 
+    @property
+    def messages_folder(self) -> str:
+        folder = os.path.join(self.log_folder, "messages")
+        os.makedirs(folder, exist_ok=True)
+        return folder
+
     @staticmethod
     def default_base_folder() -> str:
         home = os.path.expanduser("~")
@@ -1143,6 +1175,7 @@ class Tracker:
 
     def _load(self) -> None:
         self._load_scalars()
+        self._load_messages()
 
     def _load_scalars(self) -> None:
         self.scalars = {}
@@ -1151,26 +1184,55 @@ class Tracker:
             name = os.path.splitext(file)[0]
             with open(os.path.join(self.scalars_folder, file), "r") as f:
                 for line in f:
-                    data.append(float(line.strip() or "nan"))
+                    iteration, value = line.strip().split()
+                    data.append((int(iteration), float(value)))
             self.scalars[name] = data
 
+    def _load_messages(self) -> None:
+        self.messages = {}
+        for file in os.listdir(self.messages_folder):
+            name = os.path.splitext(file)[0]
+            with open(os.path.join(self.messages_folder, file), "r") as f:
+                self.messages[name] = f.read()
+
     # api
 
     def reset(self) -> None:
-        self.scalars: Dict[str, List[float]] = {}
+        self.messages: Dict[str, str] = {}
+        self.scalars: Dict[str, List[Tuple[int, float]]] = {}
 
     def track_scalar(self,
                      name: str,
-                     value: float) -> None:
+                     value: float,
+                     *,
+                     iteration: int = None) -> None:
         file = os.path.join(self.scalars_folder, f"{name}.txt")
         data = self.scalars.setdefault(name, [])
-        data.append(value)
+        if iteration is None:
+            iteration = 0 if not data else data[-1][0] + 1
+        data.append((iteration, value))
         if not os.path.isfile(file):
             with open(file, "w") as f:
-                f.write("\n".join(map(str, data)))
+                f.write("\n".join(map(lambda line: " ".join(map(str, line)), data)))
         else:
             with open(file, "a") as f:
-                f.write(f"\n{'' if math.isnan(value) else value}")
+                f.write(f"\n{iteration} {value}")
+
+    def track_message(self,
+                      name: str,
+                      message: str,
+                      *,
+                      append: bool = True) -> None:
+        file = os.path.join(self.messages_folder, f"{name}.txt")
+        existing_message = self.messages.get(name, "")
+        if not existing_message or not append:
+            self.messages[name] = message
+            with open(file, "w") as f:
+                f.write(message)
+            return
+        self.messages[name] = f"{existing_message}\n{message}"
+        with open(file, "a") as f:
+            f.write(f"\n{message}")
 
     def visualize_scalars(self,
                           types: List[str] = None,
@@ -1184,7 +1246,9 @@ class Tracker:
         plt.figure()
         for i, name in enumerate(sorted(types)):
             data = self.scalars[name]
-            plt.plot(np.arange(len(data)), data, label=name)
+            iterations, values = map(np.array, map(list, zip(*data)))
+            mask = iterations >= 0
+            plt.plot(iterations[mask], values[mask], label=name)
             if not merge:
                 plt.legend()
                 export_path = None if export_folder is None else os.path.join(export_folder, f"{name}.png")
@@ -1210,21 +1274,21 @@ class Tracker:
     def clear(self,
               *,
               confirm: bool = True) -> None:
-        print(f"clearing '{self.log_folder}'")
+        print(f"{LoggingMixin.info_prefix}clearing '{self.log_folder}'")
         if self._confirm(confirm):
             shutil.rmtree(self.log_folder)
 
     def clear_project(self,
                       *,
                       confirm: bool = True) -> None:
-        print(f"clearing '{self.project_folder}'")
+        print(f"{LoggingMixin.info_prefix}clearing '{self.project_folder}'")
         if self._confirm(confirm):
             shutil.rmtree(self.project_folder)
 
     def clear_all(self,
                   *,
                   confirm: bool = True) -> None:
-        print(f"clearing '{self.base_folder}'")
+        print(f"{LoggingMixin.info_prefix}clearing '{self.base_folder}'")
         if self._confirm(confirm):
             shutil.rmtree(self.base_folder)
 
@@ -1265,8 +1329,11 @@ class Tracker:
                 types = list(types)
             for i, name in enumerate(sorted(types)):
                 for task_name, tracker in zip(task_names, trackers):
-                    data = tracker.scalars.get(name, [])
-                    plt.plot(np.arange(len(data)), data, label=f"{name} - {task_name}")
+                    data = tracker.scalars.get(name)
+                    if data is not None:
+                        iterations, values = map(np.array, map(list, zip(*data)))
+                        mask = iterations >= 0
+                        plt.plot(iterations[mask], values[mask], label=f"{name} - {task_name}")
                 if not merge:
                     plt.legend()
                     export_path = None if export_folder is None else os.path.join(export_folder, f"{name}.png")
@@ -1284,6 +1351,6 @@ __all__ = [
     "Anneal", "Metrics", "ScalarEMA", "Visualizer", "Tracker",
     "Estimator", "ModelPattern", "EnsemblePattern", "Comparer",
     "collate_fn_type", "estimate_fn_type", "scoring_fn_type",
-    "pattern_type", "patterns_type",
+    "generic_data_type", "pattern_type", "patterns_type",
     "register_metric"
 ]
